package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.problem.AbortMethod;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.InferenceContext18;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.InvocationSite;

public abstract class NameReference extends Reference implements InvocationSite
{
    public Binding binding;
    public TypeBinding actualReceiverType;
    
    public NameReference() {
        this.bits |= 0x7;
    }
    
    @Override
    public FieldBinding fieldBinding() {
        return (FieldBinding)this.binding;
    }
    
    @Override
    public FieldBinding lastFieldBinding() {
        if ((this.bits & 0x7) == 0x1) {
            return this.fieldBinding();
        }
        return null;
    }
    
    public InferenceContext18 freshInferenceContext(Scope scope) {
        return null;
    }
    
    public boolean isSuperAccess() {
        return false;
    }
    
    public boolean isTypeAccess() {
        return this.binding == null || this.binding instanceof ReferenceBinding;
    }
    
    public boolean isTypeReference() {
        return this.binding instanceof ReferenceBinding;
    }
    
    public void setActualReceiverType(ReferenceBinding receiverType) {
        if (receiverType == null) {
            return;
        }
        this.actualReceiverType = receiverType;
        return;
    }
    
    public void setDepth(int depth) {
        this.bits &= 0xFFFFE01F;
        if (depth > 0) {
            this.bits |= (depth & 0xFF) << 5;
        }
    }
    
    public void setFieldIndex(int index) {
    }
    
    public abstract String unboundReferenceErrorName();
    
    public abstract char[][] getName();
    
    protected void checkEffectiveFinality(LocalVariableBinding localBinding, Scope scope) {
        if ((this.bits & 0x80000) != 0x0 && !localBinding.isFinal() && !localBinding.isEffectivelyFinal()) {
            scope.problemReporter().cannotReferToNonEffectivelyFinalOuterLocal(localBinding, this);
            throw new AbortMethod(scope.referenceCompilationUnit().compilationResult, null);
        }
        return;
    }
}
